/* $Id$
 * Copyright (C) 2005 Tommi Saviranta <wnd@iki.fi>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* ifdef HAVE_CONFIG_H */

#include "client.h"
#include "ftc.h"
#include "network.h"
#include "common.h"

#include <stdio.h>
#include <stdlib.h>



static int
connect_server(const char *host, int port, const char *bindhost, int bindport,
		noiselevel noise)
{
	new_connection_type *conn;
	int sock;

	conn = network_connect_init(host, port, bindhost, 1000);
	if (conn == NULL) {
		if (noise != NOISE_QUIET) {
			fprintf(stderr, "Network initialization failed!\n");
		}
		return -1;
	}

	sock = -1;
	do {
		sock = network_connect(conn);
		if (conn->error[0] != '\0' && noise == NOISE_VERBOSE) {
			printf("Connect failed: %s\n", conn->error);
		}
	} while (sock == -1 && conn->state != CONNECTION_FAILED);

	if (conn->state == CONNECTION_FAILED) {
		if (noise != NOISE_QUIET) {
			fprintf(stderr, "Connection failed: %s\n", conn->error);
		}
	} else {
		if (noise == NOISE_VERBOSE) {
			printf("Connection to %s:%d establised.\n",
					host, port);
		}
	}
	network_connect_free(conn);

	return sock;
}



int
client_get(const char *host, int port, const char *bindhost, int bindport,
		const char *pass, const char *filename, noiselevel noise)
{
	int sock;
	int r;

	sock = connect_server(host, port, bindhost, bindport, noise);
	if (sock == -1) {
		return -1;
	}
	network_socket_set_block(sock);


	r = common_get(sock, filename, noise);

	network_socket_close(sock);

	return r;
}



int
client_put(const char *host, int port, const char *bindhost, int bindport,
		const struct filelist files, int simple, const char *pass,
		noiselevel noise)
{
	int sock;
	int r;

	sock = connect_server(host, port, bindhost, bindport, noise);
	if (sock == -1) {
		return -1;
	}
	network_socket_set_block(sock);


	r = common_put(sock, files, simple, noise);

	network_socket_close(sock);

	return r;
}
