/* $Id: network.h,v 1.13 2005/09/18 11:13:19 wind Exp $
 * Copyright (C) 2005 Tommi Saviranta <wnd@iki.fi>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * This file was borrowed from miau-ng.
 */

#ifndef NETWORK_H_
#define NETWORK_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* ifdef HAVE_CONFIG_H */

#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>


#define CONNECTION_ERR_LEN	128


#define RET_BIND_OK		0
#define RET_BIND_BIND		1
#define RET_BIND_RESOLVE	2
#define RET_BIND_TIMEOUT	3

enum {
	CONNECTION_RESOLVE,
	CONNECTION_CONNECT,
	CONNECTION_ESTABLISHED,
	CONNECTION_FAILED
};

enum {
	CONNECTION_FAILED_NOFAIL,
	CONNECTION_FAILED_FATAL,
	CONNECTION_FAILED_BIND,
	CONNECTION_FAILED_RESOLV,
	CONNECTION_FAILED_CONNECT,
	CONNECTION_FAILED_SOCKET
};

enum {
	CONNECTION_IP,
	CONNECTION_NAME
};

typedef struct {
	int		state;
	int		error_state;
	
	struct addrinfo	*res;
	struct addrinfo	*res_iter;
	int		timeout;

	char		*host;
	int		port;
	char		*bind;

	char		error[CONNECTION_ERR_LEN + 1];
} new_connection_type;



int network_connect_fast(const char *host, int port, const char *bind,
		int timeout);
new_connection_type *network_connect_init(const char *host, int port,
		const char *bind, int timeout);
int network_connect(new_connection_type *conn);
void network_connect_free(new_connection_type *conn);

int network_bind_socket(int sock, const char *host, int port, int timeout);

int network_socket_new(void);
void network_socket_close(int sock);
int network_accept(int sock, struct sockaddr *sa);

int network_resolv_sockaddr(char *dest, size_t n, struct sockaddr *sa,
		size_t addr_len);

int network_socket_set_block(int sock);



#endif /* ifndef NETWORK_H_ */
