/* $Id$
 * Copyright (C) 2005 Tommi Saviranta <wnd@iki.fi>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

/* TODO: pass, use select */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* ifdef HAVE_CONFIG_H */

#include "server.h"
#include "ftc.h"
#include "network.h"
#include "common.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>



static int
create_listen(const char *host, int port, noiselevel noise)
{
	int sock;
	int r;

	/* Create socket. */

	sock = network_socket_new();
	if (sock == -1) {
		if (noise != NOISE_QUIET) {
			fprintf(stderr, "Couldn't create socket!\n");
		}
		return -1;
	}

	r = network_bind_socket(sock, host, port, 1000);
	if (r < 0) {
		if (noise != NOISE_QUIET) {
			fprintf(stderr, "Binding %s:%d failed!\n",
					host, port);
		}
		return -1;
	}

	r = listen(sock, 256);
	if (r == -1) {
		if (noise != NOISE_QUIET) {
			fprintf(stderr, "Listen failed!\n");
		}
		network_socket_close(sock);
		return -1;
	}

	return sock;
}



static int
get_client(const char *host, int port, noiselevel noise)
{
	struct sockaddr addr;
	socklen_t addrlen;
	int sock;
	int r;

	sock = create_listen(host, port, noise);
	if (sock == -1) {
		return -1;
	}

	if (noise == NOISE_VERBOSE) {
		printf("Socket created and bound. Waiting for client...\n");
	}

	/* Accept first client to connect. */
	addrlen = 0; /* keep valgrind quiet */
	r = accept(sock, &addr, &addrlen);
	if (r == -1) {
		if (noise != NOISE_QUIET) {
			fprintf(stderr, "Accept failed!\n");
		}
		network_socket_close(sock);
	} else {
		network_socket_close(sock);

		if (noise == NOISE_VERBOSE) {
			char host[BUFMAXSIZE];
			int r;
			r = network_resolv_sockaddr(host, BUFMAXSIZE,
					&addr, addrlen);
			if (r == 0) {
				printf("Client connected from %s\n", host);
			} else {
				printf("Client connected\n");
			}
		}
	}

	return r;
}



int
server_get(const char *host, int port, const char *pass, const char *filename,
		noiselevel noise)
{
	int sock;
	int r;

	sock = get_client(host, port, noise);
	if (sock == -1) {
		return -1;
	}
	network_socket_set_block(sock);
	

	r = common_get(sock, filename, noise);

	network_socket_close(sock);

	return r;
}



int
server_put(const char *host, int port, const struct filelist files,
		int simple, const char *pass, noiselevel noise)
{
	int sock;
	int r;

	sock = get_client(host, port, noise);
	if (sock == -1) {
		return -1;
	}
	network_socket_set_block(sock);


	r = common_put(sock, files, simple, noise);

	network_socket_close(sock);

	return r;
}
